
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This file contains tests involving coFJ features.
%% Usage: Type "run_tests." after having loaded the file. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% How to use the code: 
%% recall that the user is not allowed to directly create cyclic objects. 
%% They can be built by invoking a method, in the example the method repeat. For instance, the goal: 
%%
%% evaluate(invoke(new(listFact,[]),repeat,[1]),X).
%%
%% will return the cyclic object X = new(nEList, [1, X]), representing the list containing infinite instances of number 1. 
%% Note that, in this example, the role of any is crucial. 
%% Indeed, since a cycle is found, variable X (that must unify with the result of the evaluation) 
%% is unified with a list containing X as one of its fields.
%%
%% Now, we can use this object as receiver for other operations. For example, the goal:
%%
%% evaluate(invoke(new(ListFact,[]),repeat,[1]),X),
%% evaluate(invoke(X,allPos,[]),RV).
%%
%% will return the correct answer RV = true. 
%% Note that, thanks to cycle detection in the meta-interpreter, we obtain an answer even though the list is infinite.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- include(coFJ_implementation).
:- use_module(library(plunit)).
:- dynamic test_info/2.

%% Tests:

infinite_obj_test() :- evaluate(invoke(new(cycListFact,[]),infOcc,[1]),_).
new_test(X) :- evaluate(new(nEList,[1,X]),new(_,_)).
field_acc_test_1(X) :- evaluate(field_acc(X,el),_).
field_acc_test_2(X) :- evaluate(field_acc(X,nx),_).
invoke_test_1(X) :- evaluate(invoke(X,isCyc,[]),_).					
invoke_test_2(X) :- evaluate(invoke(X,allPos,[]),_).


settestinfo(Name,Fail) :- retractall(test_info(_,_)), assert(test_info(Name,Fail)).


:- begin_tests(evaluate).

%% Creates the infinite lists of ones
test(inf_obj) :- infinite_obj_test().

test(new) :- X = new(nEList,[1,X]), new_test(X).

test(field_acc_1) :- X = new(nEList,[1,X]), field_acc_test_1(X).
test(field_acc_2) :- X = new(nEList,[1,X]), field_acc_test_2(X).

%% Checks whether the infinite list of ones is cyclic
test(invoke_1) :- X = new(nEList,[1,X]), invoke_test_1(X).

%% Checks whether the infinite list of ones only contains positive numbers
test(invoke_2) :- X = new(nEList,[1,X]), invoke_test_2(X).

:- end_tests(evaluate).

%% End Tests. %%